#ifndef _LINUX_NFNETLINK_XTABLES_H
#define _LINUX_NFNETLINK_XTABLES_H 1

/**
 * %NFXTM_ERROR:	used to convey nfxt error codes
 * 			(message equipped with NFXTA_ERRNO/NFXTA_XTERRNO)
 * %NFXTM_IDENTIFY:	multifunction debug command to inquire about xt2
 * 			properties (e.g. match size)
 * %NFXTM_CHAIN_CREATE:	request creation of a chain by name
 * %NFXTM_CHAIN_DELETE:	request deletion of a chain by name
 * %NFXTM_COMMIT:	finalize and commit a transaction
 * %NFXTM_TABLE_REPLACE:start a table replace transaction
 * %NFXTM_ABORT:	abort the topmost active transaction
 * %NFXTM_CHAIN_DUMP:	retrieve chain properties and rules in the chain
 * %NFXTM_TABLE_DUMP:	retrieve table (multiple chains) and their rules
 * %NFXTM_CHAIN_SPLICE:	start a splice transaction; modify rules of a chain
 * %NFXTM_RULE_ENTRY:	used to convey rule data (during both splice and dump)
 * %NFXTM_CHAIN_PROMOTE:attach nf_hook_ops to this chain
 * %NFXTM_CHAIN_DEMOTE:	deactivate nf_hook_ops on this chain
 */
enum nfxt_msg_type {
	NFXTM_ERROR = 1,
	NFXTM_IDENTIFY,
	NFXTM_CHAIN_CREATE,
	NFXTM_CHAIN_DELETE,
	NFXTM_COMMIT,
	NFXTM_TABLE_REPLACE,
	NFXTM_ABORT,
	NFXTM_CHAIN_DUMP,
	NFXTM_TABLE_DUMP,
	NFXTM_CHAIN_SPLICE,
	NFXTM_RULE_ENTRY,
	NFXTM_CHAIN_PROMOTE,
	NFXTM_CHAIN_DEMOTE,
};

/**
 * %NFXTA_ERRNO:		system error code (%Exxx)
 * %NFXTA_XTERRNO:		NFXT-specific error code (cf. enum nfxt_errno)
 * %NFXTA_NAME:			name of the object being operated on
 * %NFXTA_CHAIN_NAME:		name of a specific chain related to the op
 * %NFXTA_REVISION_MIN:		minimum API revision supported by xtnetlink
 * %NFXTA_REVISION_MAX:		maximum API revision supported by xtnetlink
 * %NFXTA_SPLICE_OFFSET:	rule number to start deletion from
 * %NFXTA_SPLICE_DLENGTH:	delete this many rules
 * %NFXTA_HOOK_INDEX:		for base chains: the hook number
 * 				(%NF_INET_*, %NF_ARP_*, %NF_BRIDGE_*, ...)
 * %NFXTA_HOOK_PRIORITY:	(base chains:) priority for the hook
 * %NFXTA_HOOK_NFPROTO:		(base chains:) nfproto to add the hook for
 * %NFXTA_VERDICT:		verdict action in a rule
 * %NFXTA_MATCH:		container for match action
 * %NFXTA_REVISION:		revision for match/target action
 * %NFXTA_ACTION_NAME:		name of action extension
 * %NFXTA_ACTION_DATA:		opaque parameter block for match/target
 * %NFXTA_TARGET:		container for target action
 * %NFXTA_JUMP_CHAIN:		call (and return from) a chain
 * %NFXTA_GOTO_CHAIN:		go to a chain, w/o tracking return
 */
enum nfxt_attr_type {
	NFXTA_UNSPEC = 0,
	NFXTA_ERRNO,
	NFXTA_XTERRNO,
	NFXTA_NAME,
	NFXTA_CHAIN_NAME,
	NFXTA_REVISION_MIN,
	NFXTA_REVISION_MAX,
	NFXTA_SPLICE_OFFSET,
	NFXTA_SPLICE_DLENGTH,
	NFXTA_HOOK_INDEX,
	NFXTA_HOOK_PRIORITY,
	NFXTA_HOOK_NFPROTO,
	NFXTA_VERDICT,
	NFXTA_MATCH,
	NFXTA_REVISION,
	NFXTA_ACTION_NAME,
	NFXTA_ACTION_DATA,
	NFXTA_TARGET,
	NFXTA_JUMP_CHAIN,
	NFXTA_GOTO_CHAIN,
};

/**
 * %NFXTE_SUCCESS:		the operation completed successfully
 * %NFXTE_ATTRSET_INCOMPLETE:	Not all required attributes are present
 * 				in nlmsg received from userspace
 * %NFXTE_CHAIN_INVALID_NAME:	Chain name is not acceptable
 * %NFXTE_CHAIN_EXISTS:		Chain already exists
 * %NFXTE_CHAIN_NOENT:		Chain does not exist
 * %NFXTE_CHAIN_NAMETOOLONG:	New chain name is too long
 * %NFXTE_TRANSACT_ACTIVE:	Attempted to start transaction while one was
 * 				already active
 * %NFXTE_TRANSACT_INACTIVE:	Commit issued when no transaction active
 * %NFXTE_RULE_OUT_OF_RANGE:	Splice range specifies non-existent rules
 * %NFXTE_CHAIN_IS_PROMOTED:	Chain is already in promoted state
 * %NFXTE_CHAIN_IS_DEMOTED:	Chain is already in demoted state
 * %NFXTE_UNKNOWN_ATTRIBUTE:	The nlmsg contained an attribute that was not
 * 				understood and not ignored.
 * %NFXTE_ACTION_NOENT:		A match/target by that name was not found
 * %NFXTE_ACTION_MISMATCH:	Unexpected parameter block size from userspace
 * %NFXTE_ACTION_REJECTED:	Checkentry of match/target rejected parameters
 * %NFXTE_RULESET_LOOP:		The ruleset has a loop, which is not permitted
 * %NFXTE_JUMP_DST_NOENT:	The jump/goto destination does not exist
 */
enum nfxt_errno {
	NFXTE_SUCCESS = 0,
	NFXTE_ATTRSET_INCOMPLETE,
	NFXTE_CHAIN_INVALID_NAME,
	NFXTE_CHAIN_EXISTS,
	NFXTE_CHAIN_NOENT,
	NFXTE_CHAIN_NAMETOOLONG,
	NFXTE_TRANSACT_ACTIVE,
	NFXTE_TRANSACT_INACTIVE,
	NFXTE_RULE_OUT_OF_RANGE,
	NFXTE_CHAIN_IS_PROMOTED,
	NFXTE_CHAIN_IS_DEMOTED,
	NFXTE_UNKNOWN_ATTRIBUTE,
	NFXTE_ACTION_NOENT,
	NFXTE_ACTION_MISMATCH,
	NFXTE_ACTION_REJECTED,
	NFXTE_RULESET_LOOP,
	NFXTE_JUMP_DST_NOENT,
};

#endif /* _LINUX_NFNETLINK_XTABLES_H */
