#ifndef _INTERNAL_H
#define _INTERNAL_H 1

#include "config.h"
#include <stdbool.h>
#ifdef HAVE_VISIBILITY_HIDDEN
#	define EXPORT_SYMBOL __attribute__((visibility("default")))
#else
#	define EXPORT_SYMBOL
#endif
#include <libHX/defs.h>

enum xtadm_action {
	XTADM_ACTION_NONE,
	XTADM_ACTION_ADD,
	XTADM_ACTION_CHECK,
	XTADM_ACTION_DELETE,
	XTADM_ACTION_INSERT,
	XTADM_ACTION_REPLACE,
	XTADM_ACTION_SHOW,
	XTADM_ACTION_NEWCHAIN,
	XTADM_ACTION_DELCHAIN,
	XTADM_ACTION_FLUSH,
	XTADM_ACTION_PROMOTE,
	XTADM_ACTION_DEMOTE,
	XTADM_ACTION_EXECFILE,
};

enum xtadm_restore_flags {
	XTADM_RESTORE_NOFLAGS = 0,
	XTADM_RESTORE_NOFLUSH = 1 << 0,
};

struct xtnetlink_handle;
struct xtuser_rule;

struct xtadm_handle {
	struct xtnetlink_handle *xtnl_handle;
};

struct xtadm_state {
	struct xtadm_handle *adm_handle;
	enum xtadm_action action;
	unsigned int rule_num;
	bool invert;

	union {
		const char *chain_name; /* -ACDFIRSX */
		const char *exec_file; /* -B */
	};
	struct xtuser_rule *rule; /* -ACDIR */
	const char *mt_name;
	struct xtables_match *match_mod;
	struct xtables_target *target_mod;
	struct option *opts;

	/* -N, --promote */
	struct {
		const char *promote_nfproto, *promote_hook;
		const char *promote_priority;
	};
};

extern int xtadm_restore(const char *, unsigned int);
extern int xtadm_do_add(struct xtadm_state *);
extern int xtadm_do_delete(struct xtadm_state *);
extern int xtadm_do_insert(struct xtadm_state *);
extern int xtadm_do_replace(struct xtadm_state *);

/* chain.c */
extern int xtadm_do_delchain(struct xtadm_state *);
extern int xtadm_do_demote(struct xtadm_state *);
extern int xtadm_do_flush(struct xtadm_state *);
extern int xtadm_do_newchain(struct xtadm_state *);
extern int xtadm_do_promote(struct xtadm_state *);
extern int xtadm_do_show(struct xtadm_state *);

static inline struct xtnetlink_handle *xtnl(const struct xtadm_state *state)
{
	return state->adm_handle->xtnl_handle;
}

static inline void *xtadm_fake_xtentry(const void *data)
{
	/*
	 * This will naturally backfire for everything that does not
	 * limit itself to using entrymatch->data. A new function in
	 * xtables_{match,target} would be good.
	 */
	return static_cast(char *, const_cast1(void *, data)) -
	       offsetof(struct xt_entry_match, data);
}

#endif /* _INTERNAL_H */
