/*
 *	Copyright © Jan Engelhardt
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the WTF Public License version 2 or
 *	(at your option) any later version.
 */
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libxtadm/xtadm.h>
#include "config.h"

static void xtadm_usage(void)
{
	fprintf(stderr,
	PACKAGE_NAME " " PACKAGE_VERSION "\n"
	"Usage: xtadm ...\n"
//	" -A chain ...    Append a rule to a chain\n"
//	" -B file         Process rules from file\n"
//	" -C chain ...    Check existence of rule\n"
//	" -D chain ...    Delete a rule from a chain\n"
	" -F [chain]      Flush a chain or all chains\n"
//	" -I chain ...    Insert a rule to a chain\n"
	" -N chain ...    Create a new chain\n"
//	" -R chain ...    Replace a rule in a chain\n"
	" -S [chain]      Show a chain or all chains\n"
	" -X [chain]      Delete a chain or all chains\n"
	"For more commands and options, see the xtadm(8) manpage.\n"
	);
}

static void xerror(const char *s, int e)
{
	fprintf(stderr, "%s: %s\n", s, strerror(-e));
}

int main(int argc, char **argv)
{
	struct xtadm_handle *xh;
	int ret;

	ret = xtadm_init();
	if (ret < 0) {
		xerror("xtadm_init", ret);
		return EXIT_FAILURE;
	}

	ret = xtadm_open(&xh);
	if (ret < 0) {
		perror("xtadm_open");
		goto out;
	}

	ret = xtadm_execvn(xh, argc, argv);
	xtadm_close(xh);
out:
	xtadm_exit();
	if (ret == XTADM_EXIT_UNKNOWN_OPTION)
		xtadm_usage();
	return (ret == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
